/*
 * Copyright (c) 2013, Sangoma Technologies
 * James Zhang <jzhang@sangoma.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * * Neither the name of the original author; nor the names of any contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



#include "scci_listener.h"
#include "scci_session.h"


/***********************************
 *
 * APIs for user applications to call
 *
 * # start a connection
 * # register callbacks
 * # register event types which to be monitored
 * # register main event monitor thread function ( with an infinite loop to wait )
 *   If application does only provide event callbacks, then provide an default loop
 *   function to fun in lib thread.
 * # register event callback functions
 * # open socket and start listening to receive events
 *   - session started
 *   - incoming call (get call's information, session info, a-leg info)
 *   - call dropped
 *   - call answered (get b-leg info, session info)
 *   - call hangup
 *   - call destroy
 *   - hold
 *   - unhold
 *   - bridge
 *   - unbridge
 *   - appliation called
 * # close socket
 * # close a connection
 *
 * *********************************/
void scci_init_connection_data(scci_el_dispatch_callback_t disp, scci_queue_t *queue, scci_el_connection_t *conn);

int scci_start_connection(scci_el_connection_t *conn);
int scci_close_connection(scci_el_connection_t *conn);

int scci_register_callback(scci_el_connection_t *conn, scci_app_callback_handler_t cbs );


int scci_on_event_session_call_originate( scci_el_session_t *session, esl_handle_t *handle );
int scci_on_event_session_call_creat( scci_el_session_t *session, esl_handle_t *handle  );
int scci_on_event_session_call_answer( scci_el_connection_t *conn, esl_handle_t *handle );

int scci_on_event_session_call_hangup( scci_el_session_t *session, esl_handle_t *handle  );
int scci_on_event_session_call_destroy( scci_el_session_t *session, esl_handle_t *handle  );


int scci_on_event_session_call_hold( scci_el_session_t *session, esl_handle_t *handle  );
int scci_on_event_session_call_unhold( scci_el_session_t *session, esl_handle_t *handle  );
int scci_on_event_session_call_bridge( scci_el_connection_t *conn, esl_handle_t *handle, const char *dest);
int scci_on_event_session_call_execute( scci_el_connection_t *conn, esl_handle_t *handle, const char *app, const char *data);
int scci_on_event_session_call_unbridge( scci_el_session_t *session, esl_handle_t *handle  );

int scci_on_event_session_call_application( scci_el_session_t *session, esl_handle_t *handle  );
int scci_on_event_session_call_alarm( scci_el_session_t *session, esl_handle_t *handle  );



void scci_el_default_dispatch_cb(esl_socket_t server_sock, esl_socket_t client_sock, struct sockaddr_in *addr);
scci_status_t scci_app_cb_event_handler( scci_app_cb_event_t *evnt);
void scci_el_default_logger(const char *file, const char *func, int line, int level, const char *fmt, ...);


